#!/usr/bin/python3
# -*-coding:Utf-8 -*
#  version 2.1 définition des axes dans init et correction des dépréciations de scipy.linspace et du changement de
#  structure de scipy pour la fft

import sys
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import numpy as np
import scipy.signal
#  gestion du changement de module scipy
try:
    from scipy.fft import fft
except ModuleNotFoundError:
    from scipy import fft

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import matplotlib.pyplot as plt


class Window(QWidget):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)
        self.fct="Sinusoïde"
        
        #  barre de menus
        self.bar = QMenuBar()
        signal = self.bar.addMenu("Signal")
        signal.addAction("Sinusoïde")#une ligne de menu sans raccourci. Génère la str Sinusoïde
		
        carre = QAction("Carré",self)#une ligne de menu avec raccourci. (supprimé)
        #carre.setShortcut("Ctrl+S")
        signal.addAction(carre)

        ond = signal.addMenu("Onduleur")
        ond.addAction("à commande décalée")
        ond.addAction("à MLI")

        redress = QAction("Redresseur",self) 
        signal.addAction(redress)

        gradateur = signal.addMenu("Gradateur")
        gradateur.addAction("sur charge R")
        gradateur.addAction("sur charge L")

        signal.triggered[QAction].connect(self.processtrigger)

        self.setWindowTitle("Spectres de signaux usuels")

        # a figure instance to plot on, with 2 graphs
        self.figure1 = plt.figure()
        self.figure1.patch.set_facecolor('white')
        #  create an axis
        self.ax1_1 = self.figure1.add_subplot(211)

        # create an other axis
        self.ax1_2 = self.figure1.add_subplot(212)
        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas1 = FigureCanvas(self.figure1)

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbar1 = NavigationToolbar(self.canvas1, self)

        # a 2nd figure instance to plot on for MLI, with 3 graphs
        self.figure2 = plt.figure()
        self.figure2.patch.set_facecolor('white')
        # create FFT axis
        self.ax2_1 = self.figure2.add_subplot(311)

        # create time axis for MLI
        self.ax2_2 = self.figure2.add_subplot(312)

        # create time axis for porteuse et modulant
        self.ax2_3 = self.figure2.add_subplot(313)
        
        # this is the Canvas Widget that displays the `figure2`
        # it takes the `figure2` instance as a parameter to __init__
        self.canvas2 = FigureCanvas(self.figure2)

        # this is the Navigation widget
        # it takes the Canvas2 widget and a parent
        self.toolbar2 = NavigationToolbar(self.canvas2, self)
        
        #slider
        self.labelSlider = QLabel()
        self.labelSlider.setText("Nombre de\n sinusoïdes")
        
        self.slider = QSlider()
        self.MaxSlid = 10
        self.slider.setOrientation(Qt.Vertical)
        self.slider.setTickPosition(QSlider.TicksAbove)
        self.slider.setMaximum(self.MaxSlid)  # le slider est en entiers -> & correspond à 0.1ms
        self.slider.setSingleStep(1)
        self.slider.setPageStep(10)
        self.slider.setTickInterval(10)     
        self.slider.setValue(0)
        self.slider.valueChanged.connect(self.plotSlider)
        
        self.editSlider = QLineEdit()
        self.editSlider.setText("0")
        self.editSlider.setMaximumWidth(50)
        self.editSlider.textChanged.connect(self.plotLine)
        
        #  donner la fréquence du signal
        self.cadreS = QFrame()
        self.cadreS.resize(500,30)
        layoutCadreS = QHBoxLayout(self.cadreS)
        self.titreFs = QLabel()
        self.titreFs.setText("Fréquence du signal en Hz")
        self.editFs = QLineEdit('50')
        layoutCadreS.addWidget(self.titreFs)
        layoutCadreS.addWidget(self.editFs)
        
        #déco
        self.cadreS.setStyleSheet("""
        .QFrame {
            background-color: 'white';
            }
        """)
        #  donner la fréquence d'échantillonage
        self.titreFe = QLabel()
        self.titreFe.setText("Fréquence d'échantillonage en Hz")
        self.editFe = QLineEdit('10000')
        layoutCadreS.addWidget(self.titreFe)
        layoutCadreS.addWidget(self.editFe)
 
        #  donner le nombre de points d'échantillonage
        self.titreNe = QLabel()
        self.titreNe.setText("Nombre de points d'échantillonage")
        self.editNe = QLineEdit('10000')
        layoutCadreS.addWidget(self.titreNe)
        layoutCadreS.addWidget(self.editNe)
       
        #  commentaires
        self.labelC = QLabel()
        self.labelC.setText("Signal : sinusoïdes. Le curseur règle le nombre de sinusoïdes ajoutées. Choisissez le signal dans les menus déroulants")
        self.labelC.setGeometry(0,0,180,30)
        #  déco
        self.labelC.setStyleSheet("""
        .QLabel {
            border: 2px solid maroon;
            border-radius: 10px;
            background-color: 'white';
            text-align : center;
            }
        """)  #  text-align n'a aucun effet. d'où la ligne suivante
        self.labelC.setAlignment(Qt.AlignHCenter)
        
        # Just some button connected to `plot` method
        #self.button = QPushButton('Plot')
        #self.button.clicked.connect(self.plot)
        self.editFs.textChanged.connect(self.plot)
        self.editFe.textChanged.connect(self.plotNeFe)
        self.editNe.textChanged.connect(self.plotNeFe)

        # set the layout______________________________________________________________________________________
        layout = QVBoxLayout()  #  layout principal
        layoutFig = QHBoxLayout()  #  contient le graphe et le slider
        layoutSlid = QVBoxLayout()  #  contient le slider, son label et la zone d'édition
        layoutGraph = QVBoxLayout()  #  contient le graphe et ses menus
        
        layoutSlid.addWidget(self.labelSlider)
        layoutSlid.addWidget(self.slider)
        layoutSlid.addWidget(self.editSlider)
        
        self.stackFig = QStackedWidget(self)  #  création d'un stack de widgets contenant les figures ordinaires(2 courbes) et MLI (3 cbes)
        self.stackFig.addWidget(self.canvas1)
        self.stackFig.addWidget(self.canvas2)
        self.stackFig.setCurrentIndex(0)
        
        self.stackToolbar = QStackedWidget(self)  #  création d'un stack de widgets contenant les barres de menus des courbes respectives
        self.stackToolbar.setMaximumHeight(50)
        self.stackToolbar.addWidget(self.toolbar1)
        self.stackToolbar.addWidget(self.toolbar2)
        self.stackToolbar.setCurrentIndex(0)
        
        layoutGraph.addWidget(self.stackFig)
        layoutGraph.addWidget(self.stackToolbar)
        
        #  layoutFig.addWidget(self.stackFig)
        layoutFig.addLayout(layoutGraph)  #  insère layoutGraph  dans layoutFig
        layoutFig.addLayout(layoutSlid)  #  insère layoutSlid  dans layoutFig
               
        #  layout.addWidget(self.stackToolbar)
        layout.addWidget(self.bar)
        layout.addLayout(layoutFig)  #  insère layoutFig dans le layout principal
        layout.addWidget(self.labelC)
        layout.addWidget(self.cadreS)
        
        self.setLayout(layout)
        #  ____________________________________________________________________________________________________
        
        self.n = 10000  #  nombre de points d'échantillonage
        self.i = []  #  courbe du courant pour le gradateur sur charge L. Remplie dans 'fonction' en plus du paramètre 'resultat'
        self.flagEditS = False
        self.slider.setValue(1)
        self.calcPort(float(self.editFe.text()))
        self.plot()
        
    def processtrigger(self,q):
        self.fct=q.text()
        if self.fct == "Sinusoïde":
            self.MaxSlid = 10
            self.slider.setMaximum(self.MaxSlid)
            self.labelC.setText("Signal : sinusoïdes. Le curseur règle le nombre de sinusoïdes ajoutées. Choisissez le signal dans les menus déroulants")
            self.labelSlider.setText("Nombre de\n sinusoïdes")
            self.stackFig.setCurrentIndex(0)
            self.stackToolbar.setCurrentIndex(0)

        if self.fct == "Carré":
            self.labelC.setText("Signal : carré. Le curseur n'est pas utilisé.")
            self.labelSlider.setText("inutile!")
            self.stackFig.setCurrentIndex(0)
            self.stackToolbar.setCurrentIndex(0)

        if self.fct == "à commande décalée":
            self.MaxSlid = 100
            self.slider.setMaximum(self.MaxSlid)
            self.labelC.setText("Signal : à commande décalée. Le curseur règle le temps à 0.")
            self.labelSlider.setText("Temps de\n décalage\n en ms")
            self.stackFig.setCurrentIndex(0)
            self.stackToolbar.setCurrentIndex(0)

        if self.fct == "à MLI":
            self.MaxSlid = 100
            self.slider.setMaximum(self.MaxSlid)
            self.labelC.setText("Utilisez le curseur pour régler le taux de modulation. La porteuse est à 1000Hz.")
            self.labelSlider.setText("Taux de\n modulation")
            self.slider.setValue(10)
            self.editFe.setText('20000')
            self.editNe.setText('20000')
            self.stackFig.setCurrentIndex(1)
            self.stackToolbar.setCurrentIndex(1)
        
        if self.fct == "Redresseur":
            self.MaxSlid = 1
            self.slider.setMaximum(18)
            self.labelC.setText("Signal : redresseur monophasé double alternance. Le curseur règle retard à l'amorçage.")
            self.labelSlider.setText("Retard à\n l'amorçage\n en degrés")
            self.stackFig.setCurrentIndex(0)
            self.stackToolbar.setCurrentIndex(0)
        
        if self.fct == "sur charge R":
            self.MaxSlid = 1
            self.slider.setMaximum(18)
            self.labelC.setText("Signal : gradateur monophasé sur charge R. Le curseur règle retard à l'amorçage.")
            self.labelSlider.setText("Retard à\n l'amorçage\n en degrés")
            self.stackFig.setCurrentIndex(0)
            self.stackToolbar.setCurrentIndex(0)
            
        if self.fct == "sur charge L":
            self.MaxSlid = 1
            self.slider.setMaximum(18)
            self.labelC.setText("Signal : gradateur monophasé sur charge L. Courant en rouge. Le curseur règle retard à l'amorçage.")
            self.labelSlider.setText("Retard à\n l'amorçage\n en degrés")
            self.stackFig.setCurrentIndex(1)
            self.stackToolbar.setCurrentIndex(1)

        self.plotSlider()

    def calcPort(self,fe):
        Te = 1 / float(self.editFe.text())
        self.Tt = np.arange(self.n) * Te
        fp = 1000
        self.porteuse = scipy.signal.sawtooth(2 * np.pi * fp * self.Tt, 0.5)
        
    def fonction(self,f,fe):
        resultat = []
        if self.fct == "Sinusoïde":
            for t in self.Tt:
                r=0.0
                for i in range(0,int(self.slider.value())):
                    r = r+ 1/(2*i+1)*np.sin(2*np.pi*(2*i+1)*f*t)
                resultat.append(r)
            
        if self.fct == "Carré":
            for t in self.Tt:
                resultat.append(np.sign(f*(t%(1/f))-0.5))
            
        if self.fct == "à commande décalée":
            for t in self.Tt:
                T = 1/f
                reste = f*(t%T)  #  temps ramené sur une période normalisé de 0à1
                t1 = self.slider.value()/10000.0
                if reste < t1/T or (reste >= 0.5) and (reste < (0.5+t1/T)): resultat.append(0.0)
                else: resultat.append(np.sign(f*(t%(1/f))-0.5))
            
        if self.fct == "à MLI":
            for t in self.Tt:
                try:
                    if np.sin(2*np.pi*f*t)*self.slider.value()/10 >= self.porteuse[int(t*fe)]:resultat.append(1.0)
                    else: resultat.append(-1.0)
                except IndexError:
                    self.labelC.setText(str(self.porteuse.shape[-1]))
                    
        if self.fct == "Redresseur":
            for t in self.Tt:
                T = 1/f
                reste = f*(t%T)  #  temps ramené sur une période normalisé de 0à1
                t1 = self.slider.value()*5.55556e-4  #  retard à l'amorçage en s
                
                if reste <= 0.5:
                    if reste < t1/T: resultat.append(-np.sin(2*np.pi*f*t))
                    else:resultat.append(np.sin(2*np.pi*f*t))
                else:
                    if reste < (t1/T+0.5): resultat.append(np.sin(2*np.pi*f*t))
                    else: resultat.append(-np.sin(2*np.pi*f*t))
            
        if self.fct == "sur charge R":
            for t in self.Tt:
                T = 1/f
                reste = f*(t%T)  #  temps ramené sur une période normalisé de 0à1
                t1 = self.slider.value()*5.55556e-4  #  retard à l'amorçage en s
                
                if reste <= 0.5:
                    if reste < t1/T: resultat.append(0)
                    else: resultat.append(np.sin(2*np.pi*f*t))
                else:
                    if reste < (t1/T+0.5): resultat.append(0)
                    else: resultat.append(np.sin(2*np.pi*f*t))
            
        if self.fct == "sur charge L":
            for t in self.Tt:
                T = 1/f
                reste = f*(t%T)  #  temps ramené sur une période normalisé de 0à1
                t1 = self.slider.value()*5.55556e-4  #  retard à l'amorçage en s
                
                if self.slider.value() < 9:
                    resultat.append(np.sin(2*np.pi*f*t))
                    self.i.append(-0.5*np.cos(2*np.pi*f*t))
                else:
                    if reste <= 0.5-self.slider.value()/36:  #  entre 0 et pi-alpha
                        resultat.append(np.sin(2*np.pi*f*t))
                        self.i.append(0.5*(-np.cos(self.slider.value()*10*np.pi/180)-np.cos(2*np.pi*f*t)))
                    else:
                        if reste < self.slider.value()/36:  #  entre pi-alpha et alpha
                            resultat.append(0)
                            self.i.append(0)
                        else:
                            if reste < 1-self.slider.value()/36:  #  entre alpha et 2pi-alpha
                                resultat.append(np.sin(2*np.pi*f*t))
                                self.i.append(0.5*(np.cos(self.slider.value()*10*np.pi/180)-np.cos(2*np.pi*f*t)))
                            else:
                                if reste < 0.5+self.slider.value()/36:  #  entre 2pi-alpha et pi+alpha
                                    resultat.append(0)
                                    self.i.append(0)
                                else:
                                    resultat.append(np.sin(2*np.pi*f*t))
                                    self.i.append(0.5*(-np.cos(self.slider.value()*10*np.pi/180)-np.cos(2*np.pi*f*t)))
                                   
        return resultat
        
    def plotSlider(self):
        if not self.flagEditS: self.editSlider.setText(str(self.slider.value()*10/self.MaxSlid))
        self.plot()

    def plotLine(self):
        self.flagEditS = True
        self.slider.setValue(int(float(self.editSlider.text())*self.MaxSlid/10))
        self.flagEditS = False

    def plot(self):
        self.n = int(self.editNe.text())
        if self.fct == "à MLI" or self.fct == "sur charge L":self.plot3graphes()
        else:self.plot2graphes()

    def plotNeFe(self):
        self.n = int(self.editNe.text())
        self.calcPort(float(self.editFe.text()))
        if self.fct == "à MLI" or self.fct == "sur charge L" :self.plot3graphes()
        else:self.plot2graphes()

    def plot2graphes(self):
        Te = 1/float(self.editFe.text())
        self.Tt = np.arange(self.n)*Te
        
        s = []
        sSin = []
        f = float(self.editFs.text())
        nPoints = int(2/Te/f)
        s = self.fonction(f,float(self.editFe.text()))
        
        sp = 2*fft(s)/self.n
        sp[0]=sp[0]/2  #  pour obtenir la bonne amplitude de la raie continue qui n'est pas dédoublée +- comme les autres
        y=np.abs(sp)  #  calcul du module
        
        freq = np.fft.fftfreq(self.n,Te)
        nFmax = int(50*f/freq[1])

        #  courbe nulle pour marquer l'axe
        nulle = np.linspace(0.0, 0.0, num=nPoints)
        
        # discards the old graph
        self.ax1_1.clear()
        self.ax1_2.clear()
        
        # plot data        
        self.ax1_1.plot(freq[0:nFmax], y[0:nFmax])#ancienne limite self.n/2
        self.ax1_2.plot(self.Tt[:nPoints], s[:nPoints], linewidth=2)
        
        #ax2.hold(True)
        self.ax1_2.plot(self.Tt[:nPoints], nulle[:nPoints],'black')
        self.ax1_1.set_xlabel("fréquence")
        self.ax1_1.set_ylabel("amplitude")
        self.ax1_2.set_xlabel("temps")
        self.ax1_1.set_xlim([-10, 1000])#échelle horiz axe fréquences
        self.ax1_2.set_ylim([-1.1, 1.1])

        self.ax1_1.grid(True)
        self.ax1_2.grid(True)

        # refresh canvas
            
        self.canvas1.draw()

    def plot3graphes(self):
        Te = 1/float(self.editFe.text())
        self.Tt = np.arange(self.n)*Te

        s = []
        sSin = []
        f = float(self.editFs.text())
        nPoints = int(2/Te/f)
        self.i = []#initialisation de i avant calcul
        s = self.fonction(f,float(self.editFe.text()))
        sSin = np.sin(2*np.pi*f*self.Tt)*self.slider.value()/10
        sp = 2*fft(s, self.n, axis=0)/self.n
        sp[0] = sp[0]/2#pour obtenir la bonne amplitude de la raie continue qui n'est pas dédoublée +- comme les autres
        
        if self.fct == 'sur charge L':#calcul du spectre de i
            sp_i = 2*fft(self.i)/self.n
            sp_i[0] = sp_i[0]/2#pour obtenir la bonne amplitude de la raie continue qui n'est pas dédoublée +- comme les autres
            y_i = np.abs(sp_i)#calcul du module
        
        freq = np.fft.fftfreq(self.n,Te)
        nFmax = int(4000/freq[1])
        y = np.abs(sp)#calcul du module
        
        self.ax2_3.set_xlabel("temps")
        # discards the old graph
        self.ax2_1.clear()
        self.ax2_2.clear()
        self.ax2_3.clear()

        # plot data  
        if self.fct=='à MLI':#affichage du spectre de i 
            self.ax2_1.plot(freq[1:nFmax], y[1:nFmax])
            self.ax2_2.plot(self.Tt[:nPoints], sSin[:nPoints],'r')

            self.ax2_2.plot(self.Tt[:nPoints], self.porteuse[:nPoints],'b')
            self.ax2_3.plot(self.Tt[:nPoints], s[:nPoints])
            self.ax2_1.set_xlabel("fréquence")
            self.ax2_1.set_ylabel("amplitude")
            self.ax2_2.set_xlabel("temps")
            self.ax2_3.set_xlabel("temps")
            self.ax2_3.set_ylim([-1.1, 1.1])
        
        else:#gradateur sur charge L
            self.ax2_1.plot(freq[1:nFmax], y[1:nFmax])#spectre de v
            
            self.ax2_2.plot(freq[0:nFmax], y_i[0:nFmax],'red')#spectre de i
            self.ax2_3.plot(self.Tt[:nPoints], s[:nPoints])# tracé de v

            self.ax2_3.plot(self.Tt[:nPoints], self.i[:nPoints],'red',linewidth=2)# tracé de i
            
            self.ax2_1.set_xlabel("fréquence")
            self.ax2_1.set_ylabel("amplitude")
            self.ax2_2.set_xlabel("fréquence")
            self.ax2_2.set_ylabel("amplitude")
            self.ax2_3.set_xlabel("temps")
        
        self.ax2_1.grid(True)
        self.ax2_2.grid(True)
        self.ax2_3.grid(True)

        # refresh canvas
            
        self.canvas2.draw()
        #except:
            #showinfo('Données erronées !',"Entrez des nombres, séparateur décimal : le point.")


if __name__ == '__main__':
    app = QApplication(sys.argv)

    main = Window()
    main.show()
    sys.exit(app.exec_())
